<?php

namespace Drupal\synhelper\Drush;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drush\Commands\DrushCommands;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A Drush commandfile.
 */
class DrushPhpStorm extends DrushCommands {

  /**
   * The service container.
   *
   * @var \Symfony\Component\DependencyInjection\ContainerInterface
   */
  protected $container;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * MetadataGenerator constructor.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(ContainerInterface $container, EntityTypeManagerInterface $entityTypeManager) {
    parent::__construct();
    $this->container = $container;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Generate PHPStorm Metadata.
   *
   * @command synhelper:phpstorm
   *
   * @usage drush synhelper:phpstorm
   *   Generate PHPStorm Metadata.
   */
  public function generateMetadata() {
    try {
      $this->logger()->notice('Generating PHPStorm Metadata file');
      $file = $this->generate();
      file_put_contents(DRUPAL_ROOT . '/.phpstorm.meta.php', $file);
      $this->logger()->success('Done!');
    }
    catch (\Exception $e) {
      $this->logger()
        ->error("An error occurred during file generation: @error", ['@error' => $e->getMessage()]);
    }
  }

  /**
   * Generate PHPStorm metadata file contents.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function generate() {
    $this->logger()->notice('getServiceMap ...');
    $serviceMap = $this->getServiceMap();
    $this->logger()->notice('getStorageMap ...');
    $storageMap = $this->getStorageMap();
    return $this->generateFileContents($serviceMap, $storageMap);
  }

  /**
   * Generate PHPStorm metadata file contents from provided services/storages.
   *
   * @param array $serviceMap
   *   Service map.
   * @param array $storageMap
   *   Storage map.
   *
   * @return string
   *   PHPStorm metadata file contents.
   */
  private function generateFileContents(array $serviceMap, array $storageMap) {
    ob_start();
    include __DIR__ . '/.phpstorm.meta.TEMPLATE.php';
    return ob_get_clean();
  }

  /**
   * Get a map of services class names keyed by service name.
   *
   * @return array
   *   Map of services class names keyed by service name.
   */
  private function getServiceMap() {
    $serviceMap = [];
    foreach ($this->container->getServiceIds() as $serviceId) {
      try {
        $service = $this->container->get($serviceId);
      }
      catch (\Throwable $e) {
        $this->logger()->error("$serviceId ::: " . $e->getMessage());
      }
      // Skip non-existing service classes and private services.
      if (is_object($service) && strpos($serviceId, 'private__') !== 0) {
        $serviceMap[$serviceId] = '\\' . get_class($service) . '::class';
      }
    }
    return $serviceMap;
  }

  /**
   * Get a map of storage class names keyed by entity name.
   *
   * @return array
   *   Map of storage class names keyed by entity name.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getStorageMap() {
    $storageMap = [];
    foreach ($this->entityTypeManager->getDefinitions() as $type => $definition) {
      $class = $this->entityTypeManager->getStorage($type);
      $storageMap[$type] = '\\' . get_class($class) . '::class';
    }
    return $storageMap;
  }

}
